-10 alias noop
-11 alias lit
-12 alias execute
-13 alias perform
-14 alias branch-lp+!#
-15 alias branch
-16 alias ?branch
-17 alias ?branch-lp+!#
-18 alias ?dup-?branch
-19 alias ?dup-0=-?branch
-20 alias (next)
-21 alias (next)-lp+!#
-22 alias (loop)
-23 alias (loop)-lp+!#
-24 alias (+loop)
-25 alias (+loop)-lp+!#
-26 alias (-loop)
-27 alias (-loop)-lp+!#
-28 alias (s+loop)
-29 alias (s+loop)-lp+!#
-30 alias unloop
-31 alias (for)
-32 alias (do)
-33 alias (?do)
-34 alias (+do)
-35 alias (u+do)
-36 alias (-do)
-37 alias (u-do)
-38 alias i
-39 alias j
-40 alias (key)
-41 alias key?
-42 alias form
-43 alias move
-44 alias cmove
-45 alias cmove>
-46 alias fill
-47 alias compare
-48 alias -text
-49 alias capscomp
-50 alias -trailing
-51 alias /string
-52 alias +
-53 alias -
-54 alias negate
-55 alias 1+
-56 alias 1-
-57 alias max
-58 alias min
-59 alias abs
-60 alias *
-61 alias /
-62 alias mod
-63 alias /mod
-64 alias 2*
-65 alias 2/
-66 alias fm/mod
-67 alias sm/rem
-68 alias m*
-69 alias um*
-70 alias um/mod
-71 alias m+
-72 alias d+
-73 alias d-
-74 alias dnegate
-75 alias d2*
-76 alias d2/
-77 alias and
-78 alias or
-79 alias xor
-80 alias invert
-81 alias rshift
-82 alias lshift
-83 alias 0=
-84 alias 0<>
-85 alias 0<
-86 alias 0>
-87 alias 0<=
-88 alias 0>=
-89 alias =
-90 alias <>
-91 alias <
-92 alias >
-93 alias <=
-94 alias >=
-95 alias u=
-96 alias u<>
-97 alias u<
-98 alias u>
-99 alias u<=
-100 alias u>=
-101 alias d=
-102 alias d<>
-103 alias d<
-104 alias d>
-105 alias d<=
-106 alias d>=
-107 alias d0=
-108 alias d0<>
-109 alias d0<
-110 alias d0>
-111 alias d0<=
-112 alias d0>=
-113 alias du=
-114 alias du<>
-115 alias du<
-116 alias du>
-117 alias du<=
-118 alias du>=
-119 alias within
-120 alias sp@
-121 alias sp!
-122 alias rp@
-123 alias rp!
-124 alias fp@
-125 alias fp!
-126 alias ;s
-127 alias >r
-128 alias r>
-129 alias r@
-130 alias rdrop
-131 alias i'
-132 alias 2>r
-133 alias 2r>
-134 alias 2r@
-135 alias 2rdrop
-136 alias over
-137 alias drop
-138 alias swap
-139 alias dup
-140 alias rot
-141 alias -rot
-142 alias nip
-143 alias tuck
-144 alias ?dup
-145 alias pick
-146 alias 2drop
-147 alias 2dup
-148 alias 2over
-149 alias 2swap
-150 alias 2rot
-151 alias 2nip
-152 alias 2tuck
-153 alias @
-154 alias !
-155 alias +!
-156 alias c@
-157 alias c!
-158 alias 2!
-159 alias 2@
-160 alias cell+
-161 alias cells
-162 alias char+
-163 alias (chars)
-164 alias count
-165 alias (bye)
-166 alias (system)
-167 alias getenv
-168 alias open-pipe
-169 alias close-pipe
-170 alias time&date
-171 alias ms
-172 alias allocate
-173 alias free
-174 alias resize
-175 alias (f83find)
-176 alias (hashfind)
-177 alias (tablefind)
-178 alias (hashkey)
-179 alias (hashkey1)
-180 alias (parse-white)
-181 alias close-file
-182 alias open-file
-183 alias create-file
-184 alias delete-file
-185 alias rename-file
-186 alias file-position
-187 alias reposition-file
-188 alias file-size
-189 alias resize-file
-190 alias read-file
-191 alias read-line
-192 alias write-file
-193 alias emit-file
-194 alias flush-file
-195 alias file-status
-196 alias stdout
-197 alias stderr
-198 alias f=
-199 alias f<>
-200 alias f<
-201 alias f>
-202 alias f<=
-203 alias f>=
-204 alias f0=
-205 alias f0<>
-206 alias f0<
-207 alias f0>
-208 alias f0<=
-209 alias f0>=
-210 alias d>f
-211 alias f>d
-212 alias f!
-213 alias f@
-214 alias df@
-215 alias df!
-216 alias sf@
-217 alias sf!
-218 alias f+
-219 alias f-
-220 alias f*
-221 alias f/
-222 alias f**
-223 alias fnegate
-224 alias fdrop
-225 alias fdup
-226 alias fswap
-227 alias fover
-228 alias frot
-229 alias fnip
-230 alias ftuck
-231 alias float+
-232 alias floats
-233 alias floor
-234 alias fround
-235 alias fmax
-236 alias fmin
-237 alias represent
-238 alias >float
-239 alias fabs
-240 alias facos
-241 alias fasin
-242 alias fatan
-243 alias fatan2
-244 alias fcos
-245 alias fexp
-246 alias fexpm1
-247 alias fln
-248 alias flnp1
-249 alias flog
-250 alias falog
-251 alias fsin
-252 alias fsincos
-253 alias fsqrt
-254 alias ftan
-255 alias fsinh
-256 alias fcosh
-257 alias ftanh
-258 alias fasinh
-259 alias facosh
-260 alias fatanh
-261 alias sfloats
-262 alias dfloats
-263 alias aligned
-264 alias faligned
-265 alias sfaligned
-266 alias dfaligned
-267 alias >body
-268 alias >code-address
-269 alias >does-code
-270 alias code-address!
-271 alias does-code!
-272 alias does-handler!
-273 alias /does-handler
-274 alias threading-method
-275 alias flush-icache
-276 alias toupper
-277 alias @local#
-278 alias @local0
-279 alias @local1
-280 alias @local2
-281 alias @local3
-282 alias f@local#
-283 alias f@local0
-284 alias f@local1
-285 alias laddr#
-286 alias lp+!#
-287 alias lp-
-288 alias lp+
-289 alias lp+2
-290 alias lp!
-291 alias >l
-292 alias f>l
-293 alias up!
-294 alias call-c
-295 alias strerror
-296 alias strsignal
