\ the ANS Forth words and their wordsets and other info
\ anton

\ this was converted from Appendix F of the HTML version of dpANS6,
\ which was converted from the dpANS6 source (in some MS Word format)
\ by Norman Smith.

\ The format is:
\ answord word-name wordset pronounciation \ dpans6-section dpans6-page

\ I replaced the wordset name FLOATING (EXT) with FLOAT (EXT)

answord ! CORE store \ 6.1.0010 25
answord # CORE number-sign \ 6.1.0030 25
answord #> CORE number-sign-greater \ 6.1.0040 25
answord #S CORE number-sign-s \ 6.1.0050 25
answord #TIB CORE-EXT number-t-i-b \ 6.2.0060 49
answord ' CORE tick \ 6.1.0070 25
answord ( CORE paren \ 6.1.0080 26
answord ( FILE paren \ 11.6.1.0080 80
answord (LOCAL) LOCAL paren-local-paren \ 13.6.1.0086 105
answord * CORE star \ 6.1.0090 26
answord */ CORE star-slash \ 6.1.0100 26
answord */MOD CORE star-slash-mod \ 6.1.0110 26
answord + CORE plus \ 6.1.0120 26
answord +! CORE plus-store \ 6.1.0130 27
answord +LOOP CORE plus-loop \ 6.1.0140 27
answord , CORE comma \ 6.1.0150 27
answord - CORE minus \ 6.1.0160 27
answord -TRAILING STRING dash-trailing \ 17.6.1.0170 122
answord . CORE dot \ 6.1.0180 27
answord ." CORE dot-quote \ 6.1.0190 28
answord .( CORE-EXT dot-paren \ 6.2.0200 49
answord .R CORE-EXT dot-r \ 6.2.0210 49
answord .S TOOLS dot-s \ 15.6.1.0220 112
answord / CORE slash \ 6.1.0230 28
answord /MOD CORE slash-mod \ 6.1.0240 28
answord /STRING STRING slash-string \ 17.6.1.0245 123
answord 0< CORE zero-less \ 6.1.0250 28
answord 0<> CORE-EXT zero-not-equals \ 6.2.0260 49
answord 0= CORE zero-equals \ 6.1.0270 28
answord 0> CORE-EXT zero-greater \ 6.2.0280 50
answord 1+ CORE one-plus \ 6.1.0290 28
answord 1- CORE one-minus \ 6.1.0300 29
answord 2! CORE two-store \ 6.1.0310 29
answord 2* CORE two-star \ 6.1.0320 29
answord 2/ CORE two-slash \ 6.1.0330 29
answord 2>R CORE-EXT two-to-r \ 6.2.0340 50
answord 2@ CORE two-fetch \ 6.1.0350 29
answord 2CONSTANT DOUBLE two-constant \ 8.6.1.0360 66
answord 2DROP CORE two-drop \ 6.1.0370 29
answord 2DUP CORE two-dupe \ 6.1.0380 29
answord 2LITERAL DOUBLE two-literal \ 8.6.1.0390 66
answord 2OVER CORE two-over \ 6.1.0400 29
answord 2R> CORE-EXT two-r-from \ 6.2.0410 50
answord 2R@ CORE-EXT two-r-fetch \ 6.2.0415 50
answord 2ROT DOUBLE-EXT two-rote \ 8.6.2.0420 69
answord 2SWAP CORE two-swap \ 6.1.0430 30
answord 2VARIABLE DOUBLE two-variable \ 8.6.1.0440 67
answord : CORE colon \ 6.1.0450 30
answord :NONAME CORE-EXT colon-no-name \ 6.2.0455 51
answord ; CORE semicolon \ 6.1.0460 30
answord ;CODE TOOLS-EXT semicolon-code \ 15.6.2.0470 113
answord < CORE less-than \ 6.1.0480 30
answord <# CORE less-number-sign \ 6.1.0490 31
answord <> CORE-EXT not-equals \ 6.2.0500 50
answord = CORE equals \ 6.1.0530 31
answord > CORE greater-than \ 6.1.0540 31
answord >BODY CORE to-body \ 6.1.0550 31
answord >FLOAT FLOAT to-float \ 12.6.1.0558 91
answord >IN CORE to-in \ 6.1.0560 31
answord >NUMBER CORE to-number \ 6.1.0570 31
answord >R CORE to-r \ 6.1.0580 32
answord ? TOOLS question \ 15.6.1.0600 112
answord ?DO CORE-EXT question-do \ 6.2.0620 51
answord ?DUP CORE question-dupe \ 6.1.0630 32
answord @ CORE fetch \ 6.1.0650 32
answord ABORT CORE ABORT \ 6.1.0670 32
answord ABORT EXCEPTION-EXT ABORT \ 9.6.2.0670 73
answord ABORT" CORE abort-quote \ 6.1.0680 32
answord ABORT" EXCEPTION-EXT abort-quote \ 9.6.2.0680 73
answord ABS CORE abs \ 6.1.0690 32
answord ACCEPT CORE ACCEPT \ 6.1.0695 33
answord AGAIN CORE-EXT AGAIN \ 6.2.0700 51
answord AHEAD TOOLS-EXT AHEAD \ 15.6.2.0702 113
answord ALIGN CORE ALIGN \ 6.1.0705 33
answord ALIGNED CORE ALIGNED \ 6.1.0706 33
answord ALLOCATE MEMORY ALLOCATE \ 14.6.1.0707 109
answord ALLOT CORE ALLOT \ 6.1.0710 33
answord ALSO SEARCH-EXT ALSO \ 16.6.2.0715 120
answord AND CORE AND \ 6.1.0720 33
answord ASSEMBLER TOOLS-EXT ASSEMBLER \ 15.6.2.0740 114
answord AT-XY FACILITY at-x-y \ 10.6.1.0742 75
answord BASE CORE BASE \ 6.1.0750 34
answord BEGIN CORE BEGIN \ 6.1.0760 34
answord BIN FILE BIN \ 11.6.1.0765 80
answord BL CORE b-l \ 6.1.0770 34
answord BLANK STRING BLANK \ 17.6.1.0780 123
answord BLK BLOCK b-l-k \ 7.6.1.0790 62
answord BLOCK BLOCK BLOCK \ 7.6.1.0800 62
answord BUFFER BLOCK BUFFER \ 7.6.1.0820 62
answord BYE TOOLS-EXT BYE \ 15.6.2.0830 114
answord C! CORE c-store \ 6.1.0850 34
answord C" CORE-EXT c-quote \ 6.2.0855 52
answord C, CORE c-comma \ 6.1.0860 34
answord C@ CORE c-fetch \ 6.1.0870 34
answord CASE CORE-EXT CASE \ 6.2.0873 52
answord CATCH EXCEPTION CATCH \ 9.6.1.0875 72
answord CELL+ CORE cell-plus \ 6.1.0880 35
answord CELLS CORE CELLS \ 6.1.0890 35
answord CHAR CORE char \ 6.1.0895 35
answord CHAR+ CORE char-plus \ 6.1.0897 35
answord CHARS CORE chars \ 6.1.0898 35
answord CLOSE-FILE FILE CLOSE-FILE \ 11.6.1.0900 80
answord CMOVE STRING c-move \ 17.6.1.0910 123
answord CMOVE> STRING c-move-up \ 17.6.1.0920 123
answord CODE TOOLS-EXT CODE \ 15.6.2.0930 114
answord COMPARE STRING COMPARE \ 17.6.1.0935 124
answord COMPILE, CORE-EXT compile-comma \ 6.2.0945 52
answord CONSTANT CORE CONSTANT \ 6.1.0950 35
answord CONVERT CORE-EXT CONVERT \ 6.2.0970 52
answord COUNT CORE COUNT \ 6.1.0980 36
answord CR CORE c-r \ 6.1.0990 36
answord CREATE CORE CREATE \ 6.1.1000 36
answord CREATE-FILE FILE CREATE-FILE \ 11.6.1.1010 81
answord CS-PICK TOOLS-EXT c-s-pick \ 15.6.2.1015 114
answord CS-ROLL TOOLS-EXT c-s-roll \ 15.6.2.1020 115
answord D+ DOUBLE d-plus \ 8.6.1.1040 67
answord D- DOUBLE d-minus \ 8.6.1.1050 67
answord D. DOUBLE d-dot \ 8.6.1.1060 67
answord D.R DOUBLE d-dot-r \ 8.6.1.1070 67
answord D0< DOUBLE d-zero-less \ 8.6.1.1075 67
answord D0= DOUBLE d-zero-equals \ 8.6.1.1080 67
answord D2* DOUBLE d-two-star \ 8.6.1.1090 68
answord D2/ DOUBLE d-two-slash \ 8.6.1.1100 68
answord D< DOUBLE d-less-than \ 8.6.1.1110 68
answord D= DOUBLE d-equals \ 8.6.1.1120 68
answord D>F FLOAT d-to-f \ 12.6.1.1130 91
answord D>S DOUBLE d-to-s \ 8.6.1.1140 68
answord DABS DOUBLE d-abs \ 8.6.1.1160 68
answord DECIMAL CORE DECIMAL \ 6.1.1170 36
answord DEFINITIONS SEARCH DEFINITIONS \ 16.6.1.1180 119
answord DELETE-FILE FILE DELETE-FILE \ 11.6.1.1190 81
answord DEPTH CORE DEPTH \ 6.1.1200 36
answord DF! FLOAT-EXT d-f-store \ 12.6.2.1203 95
answord DF@ FLOAT-EXT d-f-fetch \ 12.6.2.1204 96
answord DFALIGN FLOAT-EXT d-f-align \ 12.6.2.1205 96
answord DFALIGNED FLOAT-EXT d-f-aligned \ 12.6.2.1207 96
answord DFLOAT+ FLOAT-EXT d-float-plus \ 12.6.2.1208 96
answord DFLOATS FLOAT-EXT d-floats \ 12.6.2.1209 96
answord DMAX DOUBLE d-max \ 8.6.1.1210 68
answord DMIN DOUBLE d-min \ 8.6.1.1220 69
answord DNEGATE DOUBLE d-negate \ 8.6.1.1230 69
answord DO CORE DO \ 6.1.1240 36
answord DOES> CORE does \ 6.1.1250 37
answord DROP CORE DROP \ 6.1.1260 37
answord DU< DOUBLE-EXT d-u-less \ 8.6.2.1270 69
answord DUMP TOOLS DUMP \ 15.6.1.1280 112
answord DUP CORE dupe \ 6.1.1290 37
answord EDITOR TOOLS-EXT EDITOR \ 15.6.2.1300 115
answord EKEY FACILITY-EXT e-key \ 10.6.2.1305 76
answord EKEY>CHAR FACILITY-EXT e-key-to-char \ 10.6.2.1306 76
answord EKEY? FACILITY-EXT e-key-question \ 10.6.2.1307 76
answord ELSE CORE ELSE \ 6.1.1310 37
answord EMIT CORE EMIT \ 6.1.1320 38
answord EMIT? FACILITY-EXT emit-question \ 10.6.2.1325 76
answord EMPTY-BUFFERS BLOCK-EXT EMPTY-BUFFERS \ 7.6.2.1330 63
answord ENDCASE CORE-EXT end-case \ 6.2.1342 53
answord ENDOF CORE-EXT end-of \ 6.2.1343 53
answord ENVIRONMENT? CORE environment-query \ 6.1.1345 38
answord ERASE CORE-EXT ERASE \ 6.2.1350 53
answord EVALUATE CORE EVALUATE \ 6.1.1360 38
answord EVALUATE BLOCK EVALUATE \ 7.6.1.1360 63
answord EXECUTE CORE EXECUTE \ 6.1.1370 38
answord EXIT CORE EXIT \ 6.1.1380 38
answord EXPECT CORE-EXT EXPECT \ 6.2.1390 53
answord F! FLOAT f-store \ 12.6.1.1400 91
answord F* FLOAT f-star \ 12.6.1.1410 91
answord F** FLOAT-EXT f-star-star \ 12.6.2.1415 96
answord F+ FLOAT f-plus \ 12.6.1.1420 91
answord F- FLOAT f-minus \ 12.6.1.1425 91
answord F. FLOAT-EXT f-dot \ 12.6.2.1427 97
answord F/ FLOAT f-slash \ 12.6.1.1430 92
answord F0< FLOAT f-zero-less-than \ 12.6.1.1440 92
answord F0= FLOAT f-zero-equals \ 12.6.1.1450 92
answord F< FLOAT f-less-than \ 12.6.1.1460 92
answord F>D FLOAT f-to-d \ 12.6.1.1470 92
answord F@ FLOAT f-fetch \ 12.6.1.1472 92
answord FABS FLOAT-EXT f-abs \ 12.6.2.1474 97
answord FACOS FLOAT-EXT f-a-cos \ 12.6.2.1476 97
answord FACOSH FLOAT-EXT f-a-cosh \ 12.6.2.1477 97
answord FALIGN FLOAT f-align \ 12.6.1.1479 92
answord FALIGNED FLOAT f-aligned \ 12.6.1.1483 92
answord FALOG FLOAT-EXT f-a-log \ 12.6.2.1484 97
answord FALSE CORE-EXT FALSE \ 6.2.1485 54
answord FASIN FLOAT-EXT f-a-sine \ 12.6.2.1486 97
answord FASINH FLOAT-EXT f-a-cinch \ 12.6.2.1487 97
answord FATAN FLOAT-EXT f-a-tan \ 12.6.2.1488 98
answord FATAN2 FLOAT-EXT f-a-tan-two \ 12.6.2.1489 98
answord FATANH FLOAT-EXT f-a-tan-h \ 12.6.2.1491 98
answord FCONSTANT FLOAT f-constant \ 12.6.1.1492 93
answord FCOS FLOAT-EXT f-cos \ 12.6.2.1493 98
answord FCOSH FLOAT-EXT f-cosh \ 12.6.2.1494 98
answord FDEPTH FLOAT f-depth \ 12.6.1.1497 93
answord FDROP FLOAT f-drop \ 12.6.1.1500 93
answord FDUP FLOAT f-dupe \ 12.6.1.1510 93
answord FE. FLOAT-EXT f-e-dot \ 12.6.2.1513 98
answord FEXP FLOAT-EXT f-e-x-p \ 12.6.2.1515 98
answord FEXPM1 FLOAT-EXT f-e-x-p-m-one \ 12.6.2.1516 99
answord FILE-POSITION FILE FILE-POSITION \ 11.6.1.1520 81
answord FILE-SIZE FILE FILE-SIZE \ 11.6.1.1522 81
answord FILE-STATUS FILE-EXT FILE-STATUS \ 11.6.2.1524 85
answord FILL CORE FILL \ 6.1.1540 39
answord FIND CORE FIND \ 6.1.1550 39
answord FIND SEARCH FIND \ 16.6.1.1550 119
answord FLITERAL FLOAT f-literal \ 12.6.1.1552 93
answord FLN FLOAT-EXT f-l-n \ 12.6.2.1553 99
answord FLNP1 FLOAT-EXT f-l-n-p-one \ 12.6.2.1554 99
answord FLOAT+ FLOAT float-plus \ 12.6.1.1555 93
answord FLOATS FLOAT FLOATS \ 12.6.1.1556 94
answord FLOG FLOAT-EXT f-log \ 12.6.2.1557 99
answord FLOOR FLOAT FLOOR \ 12.6.1.1558 94
answord FLUSH BLOCK FLUSH \ 7.6.1.1559 63
answord FLUSH-FILE FILE-EXT FLUSH-FILE \ 11.6.2.1560 85
answord FM/MOD CORE f-m-slash-mod \ 6.1.1561 39
answord FMAX FLOAT f-max \ 12.6.1.1562 94
answord FMIN FLOAT f-min \ 12.6.1.1565 94
answord FNEGATE FLOAT f-negate \ 12.6.1.1567 94
answord FORGET TOOLS-EXT FORGET \ 15.6.2.1580 115
answord FORTH SEARCH-EXT FORTH \ 16.6.2.1590 120
answord FORTH-WORDLIST SEARCH FORTH-WORDLIST \ 16.6.1.1595 119
answord FOVER FLOAT f-over \ 12.6.1.1600 94
answord FREE MEMORY FREE \ 14.6.1.1605 109
answord FROT FLOAT f-rote \ 12.6.1.1610 94
answord FROUND FLOAT f-round \ 12.6.1.1612 94
answord FS. FLOAT-EXT f-s-dot \ 12.6.2.1613 99
answord FSIN FLOAT-EXT f-sine \ 12.6.2.1614 99
answord FSINCOS FLOAT-EXT f-sine-cos \ 12.6.2.1616 100
answord FSINH FLOAT-EXT f-cinch \ 12.6.2.1617 100
answord FSQRT FLOAT-EXT f-square-root \ 12.6.2.1618 100
answord FSWAP FLOAT f-swap \ 12.6.1.1620 94
answord FTAN FLOAT-EXT f-tan \ 12.6.2.1625 100
answord FTANH FLOAT-EXT f-tan-h \ 12.6.2.1626 100
answord FVARIABLE FLOAT f-variable \ 12.6.1.1630 95
answord F~ FLOAT-EXT f-proximate \ 12.6.2.1640 100
answord GET-CURRENT SEARCH GET-CURRENT \ 16.6.1.1643 119
answord GET-ORDER SEARCH GET-ORDER \ 16.6.1.1647 119
answord HERE CORE HERE \ 6.1.1650 39
answord HEX CORE-EXT HEX \ 6.2.1660 54
answord HOLD CORE HOLD \ 6.1.1670 39
answord I CORE I \ 6.1.1680 39
answord IF CORE IF \ 6.1.1700 40
answord IMMEDIATE CORE IMMEDIATE \ 6.1.1710 40
answord INCLUDE-FILE FILE INCLUDE-FILE \ 11.6.1.1717 81
answord INCLUDED FILE INCLUDED \ 11.6.1.1718 82
answord INVERT CORE INVERT \ 6.1.1720 40
answord J CORE J \ 6.1.1730 40
answord KEY CORE KEY \ 6.1.1750 40
answord KEY? FACILITY key-question \ 10.6.1.1755 75
answord LEAVE CORE LEAVE \ 6.1.1760 41
answord LIST BLOCK-EXT LIST \ 7.6.2.1770 64
answord LITERAL CORE LITERAL \ 6.1.1780 41
answord LOAD BLOCK LOAD \ 7.6.1.1790 63
answord LOCALS| LOCAL-EXT locals-bar \ 13.6.2.1795 106
answord LOOP CORE LOOP \ 6.1.1800 41
answord LSHIFT CORE l-shift \ 6.1.1805 41
answord M* CORE m-star \ 6.1.1810 41
answord M*/ DOUBLE m-star-slash \ 8.6.1.1820 69
answord M+ DOUBLE m-plus \ 8.6.1.1830 69
answord MARKER CORE-EXT MARKER \ 6.2.1850 54
answord MAX CORE MAX \ 6.1.1870 42
answord MIN CORE MIN \ 6.1.1880 42
answord MOD CORE MOD \ 6.1.1890 42
answord MOVE CORE MOVE \ 6.1.1900 42
answord MS FACILITY-EXT MS \ 10.6.2.1905 76
answord NEGATE CORE NEGATE \ 6.1.1910 42
answord NIP CORE-EXT NIP \ 6.2.1930 54
answord OF CORE-EXT OF \ 6.2.1950 54
answord ONLY SEARCH-EXT ONLY \ 16.6.2.1965 121
answord OPEN-FILE FILE OPEN-FILE \ 11.6.1.1970 82
answord OR CORE OR \ 6.1.1980 42
answord ORDER SEARCH-EXT ORDER \ 16.6.2.1985 121
answord OVER CORE OVER \ 6.1.1990 42
answord PAD CORE-EXT PAD \ 6.2.2000 55
answord PAGE FACILITY PAGE \ 10.6.1.2005 75
answord PARSE CORE-EXT PARSE \ 6.2.2008 55
answord PICK CORE-EXT PICK \ 6.2.2030 55
answord POSTPONE CORE POSTPONE \ 6.1.2033 43
answord PRECISION FLOAT-EXT PRECISION \ 12.6.2.2035 100
answord PREVIOUS SEARCH-EXT PREVIOUS \ 16.6.2.2037 121
answord QUERY CORE-EXT QUERY \ 6.2.2040 55
answord QUIT CORE QUIT \ 6.1.2050 43
answord R/O FILE r-o \ 11.6.1.2054 82
answord R/W FILE r-w \ 11.6.1.2056 83
answord R> CORE r-from \ 6.1.2060 43
answord R@ CORE r-fetch \ 6.1.2070 43
answord READ-FILE FILE READ-FILE \ 11.6.1.2080 83
answord READ-LINE FILE READ-LINE \ 11.6.1.2090 83
answord RECURSE CORE RECURSE \ 6.1.2120 43
answord REFILL CORE-EXT REFILL \ 6.2.2125 55
answord REFILL BLOCK-EXT REFILL \ 7.6.2.2125 64
answord REFILL FILE-EXT REFILL \ 11.6.2.2125 86
answord RENAME-FILE FILE-EXT RENAME-FILE \ 11.6.2.2130 86
answord REPEAT CORE REPEAT \ 6.1.2140 44
answord REPOSITION-FILE FILE REPOSITION-FILE \ 11.6.1.2142 84
answord REPRESENT FLOAT REPRESENT \ 12.6.1.2143 95
answord RESIZE MEMORY RESIZE \ 14.6.1.2145 109
answord RESIZE-FILE FILE RESIZE-FILE \ 11.6.1.2147 84
answord RESTORE-INPUT CORE-EXT RESTORE-INPUT \ 6.2.2148 56
answord ROLL CORE-EXT ROLL \ 6.2.2150 56
answord ROT CORE rote \ 6.1.2160 44
answord RSHIFT CORE r-shift \ 6.1.2162 44
answord S" CORE s-quote \ 6.1.2165 44
answord S" FILE s-quote \ 11.6.1.2165 84
answord S>D CORE s-to-d \ 6.1.2170 44
answord SAVE-BUFFERS BLOCK SAVE-BUFFERS \ 7.6.1.2180 63
answord SAVE-INPUT CORE-EXT SAVE-INPUT \ 6.2.2182 56
answord SCR BLOCK-EXT s-c-r \ 7.6.2.2190 64
answord SEARCH STRING SEARCH \ 17.6.1.2191 124
answord SEARCH-WORDLIST SEARCH SEARCH-WORDLIST \ 16.6.1.2192 120
answord SEE TOOLS SEE \ 15.6.1.2194 112
answord SET-CURRENT SEARCH SET-CURRENT \ 16.6.1.2195 120
answord SET-ORDER SEARCH SET-ORDER \ 16.6.1.2197 120
answord SET-PRECISION FLOAT-EXT SET-PRECISION \ 12.6.2.2200 100
answord SF! FLOAT-EXT s-f-store \ 12.6.2.2202 101
answord SF@ FLOAT-EXT s-f-fetch \ 12.6.2.2203 101
answord SFALIGN FLOAT-EXT s-f-align \ 12.6.2.2204 101
answord SFALIGNED FLOAT-EXT s-f-aligned \ 12.6.2.2206 101
answord SFLOAT+ FLOAT-EXT s-float-plus \ 12.6.2.2207 101
answord SFLOATS FLOAT-EXT s-floats \ 12.6.2.2208 101
answord SIGN CORE SIGN \ 6.1.2210 45
answord SLITERAL STRING SLITERAL \ 17.6.1.2212 124
answord SM/REM CORE s-m-slash-rem \ 6.1.2214 45
answord SOURCE CORE SOURCE \ 6.1.2216 45
answord SOURCE-ID CORE-EXT source-i-d \ 6.2.2218 56
answord SOURCE-ID FILE source-i-d \ 11.6.1.2218 84
answord SPACE CORE SPACE \ 6.1.2220 45
answord SPACES CORE SPACES \ 6.1.2230 45
answord SPAN CORE-EXT SPAN \ 6.2.2240 56
answord STATE CORE STATE \ 6.1.2250 45
answord STATE TOOLS-EXT STATE \ 15.6.2.2250 115
answord SWAP CORE SWAP \ 6.1.2260 45
answord THEN CORE THEN \ 6.1.2270 46
answord THROW EXCEPTION THROW \ 9.6.1.2275 73
answord THRU BLOCK-EXT THRU \ 7.6.2.2280 64
answord TIB CORE-EXT t-i-b \ 6.2.2290 56
answord TIME&DATE FACILITY-EXT time-and-date \ 10.6.2.2292 76
answord TO CORE-EXT TO \ 6.2.2295 57
answord TO LOCAL TO \ 13.6.1.2295 106
answord TRUE CORE-EXT TRUE \ 6.2.2298 57
answord TUCK CORE-EXT TUCK \ 6.2.2300 57
answord TYPE CORE TYPE \ 6.1.2310 46
answord U. CORE u-dot \ 6.1.2320 46
answord U.R CORE-EXT u-dot-r \ 6.2.2330 57
answord U< CORE u-less-than \ 6.1.2340 46
answord U> CORE-EXT u-greater-than \ 6.2.2350 57
answord UM* CORE u-m-star \ 6.1.2360 46
answord UM/MOD CORE u-m-slash-mod \ 6.1.2370 46
answord UNLOOP CORE UNLOOP \ 6.1.2380 47
answord UNTIL CORE UNTIL \ 6.1.2390 47
answord UNUSED CORE-EXT UNUSED \ 6.2.2395 57
answord UPDATE BLOCK UPDATE \ 7.6.1.2400 63
answord VALUE CORE-EXT VALUE \ 6.2.2405 58
answord VARIABLE CORE VARIABLE \ 6.1.2410 47
answord W/O FILE w-o \ 11.6.1.2425 85
answord WHILE CORE WHILE \ 6.1.2430 47
answord WITHIN CORE-EXT WITHIN \ 6.2.2440 58
answord WORD CORE WORD \ 6.1.2450 48
answord WORDLIST SEARCH WORDLIST \ 16.6.1.2460 120
answord WORDS TOOLS WORDS \ 15.6.1.2465 113
answord WRITE-FILE FILE WRITE-FILE \ 11.6.1.2480 85
answord WRITE-LINE FILE WRITE-LINE \ 11.6.1.2485 85
answord XOR CORE x-or \ 6.1.2490 48
answord [ CORE left-bracket \ 6.1.2500 48
answord ['] CORE bracket-tick \ 6.1.2510 48
answord [CHAR] CORE bracket-char \ 6.1.2520 49
answord [COMPILE] CORE-EXT bracket-compile \ 6.2.2530 58
answord [ELSE] TOOLS-EXT bracket-else \ 15.6.2.2531 116
answord [IF] TOOLS-EXT bracket-if \ 15.6.2.2532 116
answord [THEN] TOOLS-EXT bracket-then \ 15.6.2.2533 116
answord \ CORE-EXT backslash \ 6.2.2535 58
answord \ BLOCK-EXT backslash \ 7.6.2.2535 64
answord ] CORE right-bracket \ 6.1.2540 49